package org.jenkinsci.plugins.p4_client.client;

import java.util.Properties;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perforce.p4java.PropertyDefs;
import com.perforce.p4java.exception.ConnectionException;
import com.perforce.p4java.impl.mapbased.rpc.RpcPropertyDefs;
import com.perforce.p4java.server.IOptionsServer;
import com.perforce.p4java.server.ServerFactory;

/**
 * Connection Factory
 * 
 * Provides concurrent connections to the Perforce Server
 * 
 * @author pallen
 * 
 */
public class ConnectionFactory {

	private static Logger logger = LoggerFactory
			.getLogger(ConnectionFactory.class);

	private static Identifier id = new Identifier();
	private static ConnectionConfig currentConfig;
	private static IOptionsServer connection;

	/**
	 * Returns existing connection
	 * 
	 * @return
	 */
	public static IOptionsServer getConnection() {
		return connection;
	}

	/**
	 * Provides a connection to the Perforce Server, initially client is
	 * undefined.
	 * 
	 * @param p4config
	 * @return
	 * @throws Exception
	 * @throws ConnectionException
	 * @throws Exception
	 */
	public static IOptionsServer getConnection(ConnectionConfig config)
			throws Exception {

		// Same config then return existing connection
		if (config != null && config.equals(currentConfig)) {
			if(connection.isConnected()) {
				return connection;
			}
		}

		// close old connection
		if (connection != null) {
			connection.disconnect();
		}

		// set/reset server connection
		connection = createIOptionsServer(config);
		return connection;
	}

	/**
	 * Creates a server connection
	 * 
	 * @return
	 * @throws Exception
	 */
	private static IOptionsServer createIOptionsServer(ConnectionConfig config)
			throws Exception {

		logger.info("creating IOptionsServer...");

		Properties props = System.getProperties();

		// Identify ourselves in server log files.
		props.put(PropertyDefs.PROG_NAME_KEY, id.getProduct());
		props.put(PropertyDefs.PROG_VERSION_KEY, id.getVersion());

		// Set up socket pooling to use a single socket
		props.put(RpcPropertyDefs.RPC_SOCKET_POOL_SIZE_NICK, "1");

		// Allow p4 admin commands.
		props.put(RpcPropertyDefs.RPC_RELAX_CMD_NAME_CHECKS_NICK, "true");

		// disable timeout for slow servers / large db lock times
		props.put(RpcPropertyDefs.RPC_SOCKET_SO_TIMEOUT_NICK, "0");

		// Get a server connection
		String serverUri = config.getServerUri();
		IOptionsServer iserver;
		iserver = ServerFactory.getOptionsServer(serverUri, props, null);
		iserver.connect();
		return iserver;
	}
}